--
-- SET CONSTRAINT FOR FOREIGN KEY, SET TRANSACTION and SET SESSION
-- CHARACTERISTICS

--
-- SET CONSTRAINS ALL IMMEDIATE
--
CREATE TABLE PKTABLE1 ( ptest1 int, ptest2 int, ptest3 text, PRIMARY KEY(ptest1, ptest2) );
CREATE TABLE FKTABLE1 ( ftest1 int, ftest2 int, ftest3 int, CONSTRAINT constrname FOREIGN KEY(ftest1, ftest2) 
                       REFERENCES PKTABLE1 MATCH FULL ON DELETE SET NULL ON UPDATE SET NULL INITIALLY DEFERRED);
SET CONSTRAINTS ALL IMMEDIATE;

-- Insert test data into PKTABLE1
INSERT INTO PKTABLE1 VALUES (1, 2, 'Test1');
INSERT INTO PKTABLE1 VALUES (1, 3, 'Test1-2');
INSERT INTO PKTABLE1 VALUES (2, 4, 'Test2');
INSERT INTO PKTABLE1 VALUES (3, 6, 'Test3');
INSERT INTO PKTABLE1 VALUES (4, 8, 'Test4');
INSERT INTO PKTABLE1 VALUES (5, 10, 'Test5');

-- Insert successful rows into FK TABLE1
INSERT INTO FKTABLE1 VALUES (1, 2, 4);
INSERT INTO FKTABLE1 VALUES (1, 3, 5);
INSERT INTO FKTABLE1 VALUES (2, 4, 8);
INSERT INTO FKTABLE1 VALUES (3, 6, 12);
INSERT INTO FKTABLE1 VALUES (NULL, NULL, 0);

-- Insert failed rows into FK TABLE1
INSERT INTO FKTABLE1 VALUES (100, 2, 4);
INSERT INTO FKTABLE1 VALUES (2, 2, 4);
INSERT INTO FKTABLE1 VALUES (NULL, 2, 4);
INSERT INTO FKTABLE1 VALUES (1, NULL, 4);

-- Check FKTABLE1
SELECT * FROM FKTABLE1;

-- Delete a row from PK TABLE1
DELETE FROM PKTABLE1 WHERE ptest1=1 and ptest2=2;

-- Check FKTABLE1 for removal of matched row
SELECT * FROM FKTABLE1;

-- Delete another row from PK TABLE1
DELETE FROM PKTABLE1 WHERE ptest1=5 and ptest2=10;

-- Check FKTABLE1 (should be no change)
SELECT * FROM FKTABLE1;

-- Update a row from PK TABLE1
UPDATE PKTABLE1 SET ptest1=1 WHERE ptest1=2;

-- Check FKTABLE1 for update of matched row
SELECT * FROM FKTABLE1;

DROP TABLE PKTABLE1 CASCADE;
DROP TABLE FKTABLE1;

--
-- SET CONSTRAINTS constrname2 DEFERRED
--
CREATE TABLE PKTABLE1 ( ptest1 int, ptest2 int, ptest3 text, PRIMARY KEY(ptest1, ptest2) );
CREATE TABLE FKTABLE1 ( ftest1 int DEFAULT -1, ftest2 int DEFAULT -2, ftest3 int, CONSTRAINT constrname2 FOREIGN KEY(ftest1, ftest2) 
                       REFERENCES PKTABLE1 MATCH FULL ON DELETE SET DEFAULT ON UPDATE SET DEFAULT DEFERRABLE);
SET CONSTRAINTS constrname2 DEFERRED;

BEGIN;
-- Insert a value in PKTABLE1 for default
INSERT INTO PKTABLE1 VALUES (-1, -2, 'The Default!');

-- Insert test data into PKTABLE1
INSERT INTO PKTABLE1 VALUES (1, 2, 'Test1');
INSERT INTO PKTABLE1 VALUES (1, 3, 'Test1-2');
INSERT INTO PKTABLE1 VALUES (2, 4, 'Test2');
INSERT INTO PKTABLE1 VALUES (3, 6, 'Test3');
INSERT INTO PKTABLE1 VALUES (4, 8, 'Test4');
INSERT INTO PKTABLE1 VALUES (5, 10, 'Test5');

-- Insert successful rows into FK TABLE1
INSERT INTO FKTABLE1 VALUES (1, 2, 4);
INSERT INTO FKTABLE1 VALUES (1, 3, 5);
INSERT INTO FKTABLE1 VALUES (2, 4, 8);
INSERT INTO FKTABLE1 VALUES (3, 6, 12);
INSERT INTO FKTABLE1 VALUES (NULL, NULL, 0);

COMMIT;

-- Insert failed rows into FK TABLE1
INSERT INTO FKTABLE1 VALUES (100, 2, 4);
INSERT INTO FKTABLE1 VALUES (2, 2, 4);
INSERT INTO FKTABLE1 VALUES (NULL, 2, 4);
INSERT INTO FKTABLE1 VALUES (1, NULL, 4);

-- Check FKTABLE1
SELECT * FROM FKTABLE1;

-- Delete a row from PK TABLE1
DELETE FROM PKTABLE1 WHERE ptest1=1 and ptest2=2;

-- Check FKTABLE1 to check for removal
SELECT * FROM FKTABLE1;

-- Delete another row from PK TABLE1
DELETE FROM PKTABLE1 WHERE ptest1=5 and ptest2=10;

-- Check FKTABLE1 (should be no change)
SELECT * FROM FKTABLE1;

-- Update a row from PK TABLE1
UPDATE PKTABLE1 SET ptest1=1 WHERE ptest1=2;

-- Check FKTABLE1 for update of matched row
SELECT * FROM FKTABLE1;

DROP TABLE PKTABLE1 CASCADE;
DROP TABLE FKTABLE1;



-- SET TRANSACTION ISOLATION LEVEL
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;
CREATE TABLE PKTABLE1 ( ptest1 int, ptest2 int, ptest3 int, ptest4 text, PRIMARY KEY(ptest1, ptest2, ptest3) );
CREATE TABLE FKTABLE1 ( ftest1 int, ftest2 int, ftest3 int, ftest4 int,  CONSTRAINT constrname3
			FOREIGN KEY(ftest1, ftest2, ftest3) REFERENCES PKTABLE1);

BEGIN;
-- Insert Primary Key values
INSERT INTO PKTABLE1 VALUES (1, 2, 3, 'test1');
INSERT INTO PKTABLE1 VALUES (1, 3, 3, 'test2');
INSERT INTO PKTABLE1 VALUES (2, 3, 4, 'test3');
INSERT INTO PKTABLE1 VALUES (2, 4, 5, 'test4');

-- Insert Foreign Key values
INSERT INTO FKTABLE1 VALUES (1, 2, 3, 1); 
INSERT INTO FKTABLE1 VALUES (NULL, 2, 3, 2);
INSERT INTO FKTABLE1 VALUES (2, NULL, 3, 3);
INSERT INTO FKTABLE1 VALUES (NULL, 2, 7, 4);
INSERT INTO FKTABLE1 VALUES (NULL, 3, 4, 5);

COMMIT;

-- Show FKTABLE1
SELECT * from FKTABLE1;

-- Try to update something that should succeed
UPDATE PKTABLE1 set ptest1=1 WHERE ptest2=3;

-- Try to delete something that should work
DELETE FROM PKTABLE1 where ptest1=2;

-- Show PKTABLE1 and FKTABLE1
SELECT * from PKTABLE1;

SELECT * from FKTABLE1;

DROP TABLE FKTABLE1;
DROP TABLE PKTABLE1 CASCADE;


-- SET SESSION CHARACTERISTICS AS TRANSACTION ISOLATION LEVEL READ COMMITTED
SET SESSION CHARACTERISTICS AS TRANSACTION ISOLATION LEVEL READ COMMITTED;
CREATE TABLE PKTABLE1 ( ptest1 int, ptest2 int, ptest3 int, ptest4 text, PRIMARY KEY(ptest1, ptest2, ptest3) );
CREATE TABLE FKTABLE1 ( ftest1 int, ftest2 int, ftest3 int, ftest4 int,  CONSTRAINT constrname3
			FOREIGN KEY(ftest1, ftest2, ftest3) REFERENCES PKTABLE1
			ON DELETE CASCADE ON UPDATE CASCADE);

-- Insert Primary Key values
INSERT INTO PKTABLE1 VALUES (1, 2, 3, 'test1');
INSERT INTO PKTABLE1 VALUES (1, 3, 3, 'test2');
INSERT INTO PKTABLE1 VALUES (2, 3, 4, 'test3');
INSERT INTO PKTABLE1 VALUES (2, 4, 5, 'test4');

-- Insert Foreign Key values
INSERT INTO FKTABLE1 VALUES (1, 2, 3, 1); 
INSERT INTO FKTABLE1 VALUES (NULL, 2, 3, 2);
INSERT INTO FKTABLE1 VALUES (2, NULL, 3, 3);
INSERT INTO FKTABLE1 VALUES (NULL, 2, 7, 4);
INSERT INTO FKTABLE1 VALUES (NULL, 3, 4, 5);

-- Show FKTABLE1
SELECT * from FKTABLE1;

-- Try to update something that will cascade
UPDATE PKTABLE1 set ptest2=5 where ptest2=2;

-- Try to update something that should not cascade
UPDATE PKTABLE1 set ptest1=1 WHERE ptest2=3;

-- Show PKTABLE1 and FKTABLE1
SELECT * from PKTABLE1;
SELECT * from FKTABLE1;

-- Try to delete something that should cascade
DELETE FROM PKTABLE1 where ptest1=1 and ptest2=5 and ptest3=3;

-- Show PKTABLE1 and FKTABLE1
SELECT * from PKTABLE1;
SELECT * from FKTABLE1;

-- Try to delete something that should not have a cascade
DELETE FROM PKTABLE1 where ptest1=2;

-- Show PKTABLE1 and FKTABLE1
SELECT * from PKTABLE1;
SELECT * from FKTABLE1;

DROP TABLE FKTABLE1;
DROP TABLE PKTABLE1 CASCADE;
